@@ -23,31 +23,61 @@ mod cast;
 mod error;
 mod if_expr;
 
+mod bitwise_not;
+pub use bitwise_not::{bitwise_not, BitwiseNotExpr};
+mod checkoverflow;
+pub use checkoverflow::CheckOverflow;
+
 mod kernels;
 mod list;
-mod regexp;
 pub mod scalar_funcs;
-pub mod spark_hash;
-mod structs;
-mod temporal;
-pub mod timezone;
-mod to_json;
-pub mod utils;
+mod schema_adapter;
+mod static_invoke;
+pub use schema_adapter::SparkSchemaAdapterFactory;
+pub use static_invoke::*;
 
-pub use cast::{spark_cast, Cast};
+mod negative;
+mod struct_funcs;
+pub use negative::{create_negate_expr, NegativeExpr};
+mod normalize_nan;
+
+mod json_funcs;
+pub mod test_common;
+pub mod timezone;
+mod unbound;
+pub use unbound::UnboundColumn;
+pub mod utils;
+pub use normalize_nan::NormalizeNaNAndZero;
+mod predicate_funcs;
+pub use predicate_funcs::{spark_isnan, RLike};
+
+mod agg_funcs;
+mod comet_scalar_funcs;
+pub mod hash_funcs;
+
+mod string_funcs;
+
+mod datetime_funcs;
+pub use agg_funcs::*;
+
+pub use crate::{CreateNamedStruct, GetStructField};
+pub use crate::{DateTruncExpr, HourExpr, MinuteExpr, SecondExpr, TimestampTruncExpr};
+pub use cast::{spark_cast, Cast, SparkCastOptions};
+pub use comet_scalar_funcs::create_comet_physical_fun;
+pub use datetime_funcs::*;
 pub use error::{SparkError, SparkResult};
+pub use hash_funcs::*;
 pub use if_expr::IfExpr;
-pub use list::{GetArrayStructFields, ListExtract};
-pub use regexp::RLike;
-pub use structs::{CreateNamedStruct, GetStructField};
-pub use temporal::{DateTruncExpr, HourExpr, MinuteExpr, SecondExpr, TimestampTruncExpr};
-pub use to_json::ToJson;
+pub use json_funcs::ToJson;
+pub use list::{ArrayInsert, GetArrayStructFields, ListExtract};
+pub use string_funcs::*;
+pub use struct_funcs::*;
 
 /// Spark supports three evaluation modes when evaluating expressions, which affect
 /// the behavior when processing input values that are invalid or would result in an
 /// error, such as divide by zero errors, and also affects behavior when converting
 /// between types.
-#[derive(Debug, Hash, PartialEq, Clone, Copy)]
+#[derive(Debug, Hash, PartialEq, Eq, Clone, Copy)]
 pub enum EvalMode {
     /// Legacy is the default behavior in Spark prior to Spark 4.0. This mode silently ignores
     /// or replaces errors during SQL operations. Operations resulting in errors (like
@@ -61,3 +91,9 @@ pub enum EvalMode {
     /// failing the entire query.
     Try,
 }
+
+pub(crate) fn arithmetic_overflow_error(from_type: &str) -> SparkError {
+    SparkError::ArithmeticOverflow {
+        from_type: from_type.to_string(),
+    }
+}
